# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from six import with_metaclass
import django.core.exceptions
import django.db.models
import django.forms
from django.utils.encoding import smart_text
from . import fields


class SouthMixinBase(object):
    south_field_class = ''

    def south_field_triple(self):
        """Returns a suitable description of this field for South."""
        if not self.south_field_class:
            raise NotImplementedError('please set south_field_class when using the south field mixin')
        # We'll just introspect ourselves, since we inherit.
        from south.modelsinspector import introspector
        field_class = self.south_field_class
        args, kwargs = introspector(self)
        # That's our definition!
        return field_class, args, kwargs


class SouthCharFieldMixin(SouthMixinBase):
    south_field_class = "django.db.models.fields.CharField"


class SouthTextFieldMixin(SouthMixinBase):
    south_field_class = "django.db.models.fields.TextField"


class SouthIntegerFieldMixin(SouthMixinBase):
    south_field_class = "django.db.models.fields.IntegerField"


class Classes(django.db.models.TextField, SouthTextFieldMixin):
    # TODO: validate
    default_field_class = fields.Classes

    def __init__(self, *args, **kwargs):
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        if 'default' not in kwargs:
            kwargs['default'] = ''
        if 'help_text' not in kwargs:
            kwargs['help_text'] = 'space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>'
        super(Classes, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Classes, self).formfield(**defaults)


class Context(django.db.models.fields.CharField, SouthCharFieldMixin):
    default_field_class = fields.Context

    def __init__(self, *args, **kwargs):
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 255
        if 'blank' not in kwargs:
            kwargs['blank'] = False
        if 'default' not in kwargs:
            kwargs['default'] = self.default_field_class.DEFAULT
        super(Context, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
            'choices_form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Context, self).formfield(**defaults)

    def get_choices(self, **kwargs):
        # if there already is a "blank" choice, don't add another
        # default blank choice
        if '' in dict(self.choices).keys():
            kwargs['include_blank'] = False
        return super(Context, self).get_choices(**kwargs)


class Size(django.db.models.CharField, SouthCharFieldMixin):
    default_field_class = fields.Size

    def __init__(self, *args, **kwargs):
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 255
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        if 'default' not in kwargs:
            kwargs['default'] = self.default_field_class.DEFAULT
        super(Size, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
            'choices_form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Size, self).formfield(**defaults)

    def get_choices(self, **kwargs):
        # if there already is a "blank" choice, don't add another
        # default blank choice
        if '' in dict(self.choices).keys():
            kwargs['include_blank'] = False
        return super(Size, self).get_choices(**kwargs)


class Icon(django.db.models.CharField, SouthCharFieldMixin):
    default_field_class = fields.Icon

    def __init__(self, *args, **kwargs):
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 255
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        if 'default' not in kwargs:
            kwargs['default'] = self.default_field_class.DEFAULT
        super(Icon, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Icon, self).formfield(**defaults)


class IntegerField(django.db.models.IntegerField, SouthIntegerFieldMixin):
    default_field_class = fields.Integer

    def __init__(self, verbose_name=None, name=None, min_value=None, max_value=None, **kwargs):
        self.min_value, self.max_value = min_value, max_value
        django.db.models.IntegerField.__init__(self, verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
            'min_value': self.min_value,
            'max_value': self.max_value,
        }
        defaults.update(kwargs)
        return super(IntegerField, self).formfield(**defaults)


class MiniText(django.db.models.TextField, SouthTextFieldMixin):
    default_field_class = fields.MiniText

    def __init__(self, *args, **kwargs):
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        if 'default' not in kwargs:
            kwargs['default'] = ''
        super(MiniText, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(MiniText, self).formfield(**defaults)


class LinkOrButton(django.db.models.fields.CharField, SouthCharFieldMixin):
    default_field_class = fields.LinkOrButton

    def __init__(self, *args, **kwargs):
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 10
        if 'blank' not in kwargs:
            kwargs['blank'] = False
        if 'default' not in kwargs:
            kwargs['default'] = self.default_field_class.DEFAULT
        super(LinkOrButton, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
            'choices_form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(LinkOrButton, self).formfield(**defaults)

    def get_choices(self, **kwargs):
        # if there already is a "blank" choice, don't add another
        # default blank choice
        if '' in dict(self.choices).keys():
            kwargs['include_blank'] = False
        return super(LinkOrButton, self).get_choices(**kwargs)


# class JSONField(json_field.JSONField, SouthTextFieldMixin):
#     pass


class Responsive(MiniText):
    default_field_class = fields.Responsive

    def __init__(self, *args, **kwargs):
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        if 'default' not in kwargs:
            kwargs['default'] = ''
        super(Responsive, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Responsive, self).formfield(**defaults)


class ResponsivePrint(MiniText):
    default_field_class = fields.ResponsivePrint

    def __init__(self, *args, **kwargs):
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        if 'default' not in kwargs:
            kwargs['default'] = ''
        super(ResponsivePrint, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(ResponsivePrint, self).formfield(**defaults)


#TODO:
#   * btn-block, disabled
#   * pull-left, pull-right
#   * margins/padding



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import django.forms.fields
from django.utils.translation import ugettext_lazy as _
from . import widgets, constants
from .conf import settings


class Context(django.forms.fields.ChoiceField):
    widget = widgets.Context
    CHOICES = constants.CONTEXT_CHOICES
    DEFAULT = constants.CONTEXT_DEFAULT

    def __init__(self, *args, **kwargs):
        if 'choices' not in kwargs:
            kwargs['choices'] = self.CHOICES
        if 'initial' not in kwargs:
            kwargs['initial'] = self.DEFAULT
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(Context, self).__init__(*args, **kwargs)


class Size(django.forms.fields.ChoiceField):
    widget = widgets.Size
    CHOICES = constants.SIZE_WIDGET_CHOICES
    DEFAULT = constants.SIZE_WIDGET_DEFAULT

    def __init__(self, *args, **kwargs):
        if 'choices' not in kwargs:
            kwargs['choices'] = self.CHOICES
        if 'initial' not in kwargs:
            kwargs['initial'] = self.DEFAULT
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(Size, self).__init__(*args, **kwargs)


class Classes(django.forms.fields.CharField):
    pass


class Icon(django.forms.fields.CharField):
    widget = widgets.Icon
    DEFAULT = ''

    def __init__(self, *args, **kwargs):
        if 'initial' not in kwargs:
            kwargs['initial'] = self.DEFAULT
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(Icon, self).__init__(*args, **kwargs)


class Integer(django.forms.fields.IntegerField):
    widget = django.forms.NumberInput

    def __init__(self, *args, **kwargs):
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(Integer, self).__init__(*args, **kwargs)


class Classes(django.forms.fields.CharField):
    widget = django.forms.widgets.Textarea


class MiniText(django.forms.fields.CharField):
    widget = widgets.MiniTextarea

    def __init__(self, *args, **kwargs):
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(MiniText, self).__init__(*args, **kwargs)


class LinkOrButton(django.forms.fields.ChoiceField):
    widget = widgets.LinkOrButton
    CHOICES = (
        ('lnk', 'link'),
        ('btn', 'button'),
    )
    DEFAULT = 'lnk'

    def __init__(self, *args, **kwargs):
        if 'choices' not in kwargs:
            kwargs['choices'] = self.CHOICES
        if 'initial' not in kwargs:
            kwargs['initial'] = self.DEFAULT
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(LinkOrButton, self).__init__(*args, **kwargs)


class Responsive(MiniText):
    widget = widgets.Responsive


class ResponsivePrint(MiniText):
    widget = widgets.ResponsivePrint



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import aldryn_bootstrap3.model_fields


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_bootstrap3', '0002_bootstrap3fileplugin'),
    ]

    operations = [
        migrations.AddField(
            model_name='boostrap3imageplugin',
            name='override_height',
            field=models.IntegerField(help_text='if this field is provided - it would be used across all devices instead of default for devices types. If aspect ration is selected - height will be calculated based on that.', null=True, verbose_name='override height', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='boostrap3imageplugin',
            name='override_width',
            field=models.IntegerField(help_text='if this field is provided - it would be used across all devices instead of default for devices types.', null=True, verbose_name='override width', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='boostrap3buttonplugin',
            name='btn_context',
            field=aldryn_bootstrap3.model_fields.Context(default='default', max_length=255, verbose_name='context', choices=[('default', 'Default'), ('primary', 'Primary'), ('success', 'Success'), ('info', 'Info'), ('warning', 'Warning'), ('danger', 'Danger'), ('link', 'Link')]),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='boostrap3buttonplugin',
            name='link_mailto',
            field=models.EmailField(max_length=75, null=True, verbose_name='email address', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='boostrap3buttonplugin',
            name='txt_context',
            field=aldryn_bootstrap3.model_fields.Context(default='', max_length=255, verbose_name='context', blank=True, choices=[('', 'Default'), ('primary', 'Primary'), ('success', 'Success'), ('info', 'Info'), ('warning', 'Warning'), ('danger', 'Danger'), ('muted ', 'Muted')]),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='boostrap3imageplugin',
            name='aspect_ratio',
            field=models.CharField(default='', max_length=10, verbose_name='aspect ratio', blank=True, choices=[('1x1', '1x1'), ('4x3', '4x3'), ('16x9', '16x9'), ('16x10', '16x10'), ('21x9', '21x9'), ('3x4', '3x4'), ('9x16', '9x16'), ('10x16', '10x16'), ('9x21', '9x21')]),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='bootstrap3carouselplugin',
            name='aspect_ratio',
            field=models.CharField(default='', max_length=10, verbose_name='aspect ratio', blank=True, choices=[('1x1', '1x1'), ('4x3', '4x3'), ('16x9', '16x9'), ('16x10', '16x10'), ('21x9', '21x9'), ('3x4', '3x4'), ('9x16', '9x16'), ('10x16', '10x16'), ('9x21', '9x21')]),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='bootstrap3carouselslideplugin',
            name='link_mailto',
            field=models.EmailField(max_length=75, null=True, verbose_name='email address', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='bootstrap3listgroupitemplugin',
            name='context',
            field=aldryn_bootstrap3.model_fields.Context(default='', max_length=255, blank=True, choices=[('', 'Default'), ('primary', 'Primary'), ('success', 'Success'), ('info', 'Info'), ('warning', 'Warning'), ('danger', 'Danger')]),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from appconf import AppConf


class AldrynSitesConf(AppConf):
    GRID_SIZE = 24
    ICONSETS = ( # NOTE: these values are overridden by the settings from aldryn_config.py on aldryn
        # first value is the iconset identifier for http://victor-valencia.github.io/bootstrap-iconpicker/
        # second is the prefix for the css class
        # third is the pretty name shown in the select box
        ('glyphicons', 'glyphicons', 'Glyphicons'),
        ('fontawesome', 'fa', 'Fontawesome'),
    )
